@@ -1,7 +1,8 @@
 #include <RcppEigen.h>
 
+#include "RNifti.h"
+
 #include "config.h"
-#include "NiftiImage.h"
 #include "DeformationField.h"
 #include "aladin.h"
 #include "f3d.h"
@@ -15,125 +16,10 @@ using namespace Rcpp;
 
 typedef std::vector<float> float_vector;
 
-std::map<std::string,short> NiftiImage::DatatypeCodes = NiftiImage::buildDatatypeCodes();
-
-RcppExport SEXP retrieveImage (SEXP _image)
+RcppExport SEXP initialise ()
 {
 BEGIN_RCPP
-    NiftiImage image(_image);
-    return image.toPointer("NIfTI image");
-END_RCPP
-}
-
-RcppExport SEXP readNifti (SEXP _file, SEXP _internal)
-{
-BEGIN_RCPP
-    NiftiImage image(_file);
-    return image.toArrayOrPointer(as<bool>(_internal), "NIfTI image");
-END_RCPP
-}
-
-RcppExport SEXP writeNifti (SEXP _image, SEXP _file, SEXP _datatype)
-{
-BEGIN_RCPP
-    NiftiImage image(_image);
-    std::string datatypeString = as<std::string>(_datatype);
-    if (NiftiImage::DatatypeCodes.count(datatypeString) == 0)
-    {
-        std::ostringstream message;
-        message << "Datatype \"" << datatypeString << "\" is not valid";
-        Rf_warning(message.str().c_str());
-        
-        datatypeString = "auto";
-    }
-    image.toFile(as<std::string>(_file), NiftiImage::DatatypeCodes[datatypeString]);
-    return R_NilValue;
-END_RCPP
-}
-
-RcppExport SEXP updateNifti (SEXP _image, SEXP _reference)
-{
-BEGIN_RCPP
-    const NiftiImage reference(_reference);
-    RObject image(_image);
-    
-    if (!reference.isNull())
-    {
-        NiftiImage updatedImage = reference;
-        updatedImage.update(image);
-        return updatedImage.toArray();
-    }
-    else
-        return image;
-END_RCPP
-}
-
-RcppExport SEXP dumpNifti (SEXP _image)
-{
-BEGIN_RCPP
-    const NiftiImage image(_image, false);
-    return image.headerToList();
-END_RCPP
-}
-
-RcppExport SEXP getXform (SEXP _image, SEXP _preferQuaternion)
-{
-BEGIN_RCPP
-    const NiftiImage image(_image, false);
-    const bool preferQuaternion = as<bool>(_preferQuaternion);
-    
-    AffineMatrix matrix(image.xform(preferQuaternion), false);
-    return matrix;
-END_RCPP
-}
-
-RcppExport SEXP setXform (SEXP _image, SEXP _matrix, SEXP _isQform)
-{
-BEGIN_RCPP
-    NiftiImage image(_image);
-    AffineMatrix matrix(_matrix);
-    
-    int code = -1;
-    if (!Rf_isNull(matrix.attr("code")))
-        code = as<int>(matrix.attr("code"));
-    
-    if (!image.isNull())
-    {
-        if (as<bool>(_isQform))
-        {
-            image->qto_xyz = mat44(matrix);
-            image->qto_ijk = nifti_mat44_inverse(image->qto_xyz);
-            nifti_mat44_to_quatern(image->qto_xyz, &image->quatern_b, &image->quatern_c, &image->quatern_d, &image->qoffset_x, &image->qoffset_y, &image->qoffset_z, NULL, NULL, NULL, &image->qfac);
-            
-            if (code >= 0)
-                image->qform_code = code;
-        }
-        else
-        {
-            image->sto_xyz = mat44(matrix);
-            image->sto_ijk = nifti_mat44_inverse(image->sto_xyz);
-            
-            if (code >= 0)
-                image->sform_code = code;
-        }
-    }
-    
-    if (image.isPersistent())
-        return _image;
-    else
-        return image.toArray();
-END_RCPP
-}
-
-RcppExport SEXP rescaleImage (SEXP _image, SEXP _scales)
-{
-BEGIN_RCPP
-    const std::vector<float> scales = as<float_vector>(_scales);
-    const NiftiImage image(_image);
-    
-    NiftiImage newImage(nifti_copy_nim_info(image));
-    newImage.rescale(scales);
-    return newImage.toPointer("NIfTI image");
+    niftilib_register_all();
 END_RCPP
 }
 
@@ -144,6 +30,9 @@ void checkImages (const NiftiImage &sourceImage, const NiftiImage &targetImage)
     if (targetImage.isNull())
         throw std::runtime_error("Cannot read or retrieve target image");
     
+    reg_checkAndCorrectDimension(sourceImage);
+    reg_checkAndCorrectDimension(targetImage);
+    
     const int nSourceDim = sourceImage.nDims();
     const int nTargetDim = targetImage.nDims();
     
@@ -197,6 +86,27 @@ BEGIN_RCPP
 END_RCPP
 }
 
+NiftiImage allocateMultiregResult (const NiftiImage &source, const NiftiImage &target, const bool forceDouble)
+{
+    nifti_image *newStruct = nifti_copy_nim_info(target);
+    newStruct->dim[0] = source->dim[0];
+    newStruct->dim[source.nDims()] = source->dim[source.nDims()];
+    newStruct->pixdim[source.nDims()] = source->pixdim[source.nDims()];
+    
+    if (forceDouble)
+    {
+        newStruct->datatype = DT_FLOAT64;
+        nifti_datatype_sizes(newStruct->datatype, &newStruct->nbyper, NULL);
+    }
+    
+    nifti_update_dims_from_array(newStruct);
+    
+    size_t dataSize = nifti_get_volsize(newStruct);
+    newStruct->data = calloc(1, dataSize);
+    
+    return NiftiImage(newStruct);
+}
+
 RcppExport SEXP regLinear (SEXP _source, SEXP _target, SEXP _type, SEXP _symmetric, SEXP _nLevels, SEXP _maxIterations, SEXP _useBlockPercentage, SEXP _interpolation, SEXP _sourceMask, SEXP _targetMask, SEXP _init, SEXP _verbose, SEXP _estimateOnly, SEXP _sequentialInit, SEXP _internal)
 {
 BEGIN_RCPP
@@ -213,6 +123,10 @@ BEGIN_RCPP
     const bool estimateOnly = as<bool>(_estimateOnly);
     const bool sequentialInit = as<bool>(_sequentialInit);
     
+    const int internal = as<int>(_internal);
+    const bool internalOutput = (internal == TRUE);
+    const bool internalInput = (internal != FALSE);
+    
     List init(_init);
     List returnValue;
     
@@ -226,15 +140,15 @@ BEGIN_RCPP
     
         AladinResult result = regAladin(sourceImage, targetImage, scope, symmetric, as<int>(_nLevels), as<int>(_maxIterations), as<int>(_useBlockPercentage), as<int>(_interpolation), sourceMask, targetMask, initAffine, as<bool>(_verbose), estimateOnly);
         
-        returnValue["image"] = result.image.toArrayOrPointer(as<bool>(_internal), "Result image");
+        returnValue["image"] = result.image.toArrayOrPointer(internalOutput, "Result image");
         returnValue["forwardTransforms"] = List::create(result.forwardTransform);
         if (symmetric)
             returnValue["reverseTransforms"] = List::create(result.reverseTransform);
         else
             returnValue["reverseTransforms"] = R_NilValue;
         returnValue["iterations"] = List::create(result.iterations);
-        returnValue["source"] = List::create(sourceImage.toPointer("Source image"));
-        returnValue["target"] = targetImage.toPointer("Target image");
+        returnValue["source"] = List::create(sourceImage.toArrayOrPointer(internalInput, "Source image"));
+        returnValue["target"] = targetImage.toArrayOrPointer(internalInput, "Target image");
         
         return returnValue;
     }
@@ -251,7 +165,7 @@ BEGIN_RCPP
                 currentSource = sourceImage.slice(i);
             else
                 currentSource = sourceImage.volume(i);
-            sourceImages[i] = currentSource.toPointer("Source image");
+            sourceImages[i] = currentSource.toArrayOrPointer(internalInput, "Source image");
             
             AffineMatrix initAffine;
             if (!Rf_isNull(init[i]))
@@ -274,7 +188,7 @@ BEGIN_RCPP
             iterations[i] = result.iterations;
         }
         
-        returnValue["image"] = finalImage.toArrayOrPointer(as<bool>(_internal), "Result image");
+        returnValue["image"] = finalImage.toArrayOrPointer(internalOutput, "Result image");
         returnValue["forwardTransforms"] = forwardTransforms;
         if (symmetric)
             returnValue["reverseTransforms"] = reverseTransforms;
@@ -282,7 +196,7 @@ BEGIN_RCPP
             returnValue["reverseTransforms"] = R_NilValue;
         returnValue["iterations"] = iterations;
         returnValue["source"] = sourceImages;
-        returnValue["target"] = targetImage.toPointer("Target image");
+        returnValue["target"] = targetImage.toArrayOrPointer(internalInput, "Target image");
         
         return returnValue;
     }
@@ -312,6 +226,10 @@ BEGIN_RCPP
     const bool estimateOnly = as<bool>(_estimateOnly);
     const bool sequentialInit = as<bool>(_sequentialInit);
     
+    const int internal = as<int>(_internal);
+    const bool internalOutput = (internal == TRUE);
+    const bool internalInput = (internal != FALSE);
+    
     List init(_init);
     List returnValue;
     
@@ -333,15 +251,15 @@ BEGIN_RCPP
     
         F3dResult result = regF3d(sourceImage, targetImage, as<int>(_nLevels), as<int>(_maxIterations), interpolation, sourceMask, targetMask, initControl, initAffine, as<int>(_nBins), as<float_vector>(_spacing), as<float>(_bendingEnergyWeight), as<float>(_linearEnergyWeight), as<float>(_jacobianWeight), symmetric, as<bool>(_verbose), estimateOnly);
         
-        returnValue["image"] = result.image.toArrayOrPointer(as<bool>(_internal), "Result image");
-        returnValue["forwardTransforms"] = List::create(result.forwardTransform.toPointer("F3D control points"));
+        returnValue["image"] = result.image.toArrayOrPointer(internalOutput, "Result image");
+        returnValue["forwardTransforms"] = List::create(result.forwardTransform.toArrayOrPointer(internalInput, "F3D control points"));
         if (symmetric)
-            returnValue["reverseTransforms"] = List::create(result.reverseTransform.toPointer("F3D control points"));
+            returnValue["reverseTransforms"] = List::create(result.reverseTransform.toArrayOrPointer(internalInput, "F3D control points"));
         else
             returnValue["reverseTransforms"] = R_NilValue;
         returnValue["iterations"] = List::create(result.iterations);
-        returnValue["source"] = List::create(sourceImage.toPointer("Source image"));
-        returnValue["target"] = targetImage.toPointer("Target image");
+        returnValue["source"] = List::create(sourceImage.toArrayOrPointer(internalInput, "Source image"));
+        returnValue["target"] = targetImage.toArrayOrPointer(internalInput, "Target image");
         
         return returnValue;
     }
@@ -358,7 +276,7 @@ BEGIN_RCPP
                 currentSource = sourceImage.slice(i);
             else
                 currentSource = sourceImage.volume(i);
-            sourceImages[i] = currentSource.toPointer("Source image");
+            sourceImages[i] = currentSource.toArrayOrPointer(internalInput, "Source image");
             
             AffineMatrix initAffine;
             NiftiImage initControl;
@@ -383,13 +301,13 @@ BEGIN_RCPP
             else
                 finalImage.volume(i) = result.image;
             
-            forwardTransforms[i] = result.forwardTransform.toPointer("F3D control points");
+            forwardTransforms[i] = result.forwardTransform.toArrayOrPointer(internalInput, "F3D control points");
             if (symmetric)
-                reverseTransforms[i] = result.reverseTransform.toPointer("F3D control points");
+                reverseTransforms[i] = result.reverseTransform.toArrayOrPointer(internalInput, "F3D control points");
             iterations[i] = result.iterations;
         }
         
-        returnValue["image"] = finalImage.toArrayOrPointer(as<bool>(_internal), "Result image");
+        returnValue["image"] = finalImage.toArrayOrPointer(internalOutput, "Result image");
         returnValue["forwardTransforms"] = forwardTransforms;
         if (symmetric)
             returnValue["reverseTransforms"] = reverseTransforms;
@@ -397,7 +315,7 @@ BEGIN_RCPP
             returnValue["reverseTransforms"] = R_NilValue;
         returnValue["iterations"] = iterations;
         returnValue["source"] = sourceImages;
-        returnValue["target"] = targetImage.toPointer("Target image");
+        returnValue["target"] = targetImage.toArrayOrPointer(internalInput, "Target image");
         
         return returnValue;
     }
