@@ -471,7 +471,7 @@ void reg_f3d_sym<T>::Initialise()
 
    if(this->inputControlPointGrid==NULL){
       // Define the spacing for the first level
-      float gridSpacing[3] = {this->spacing[0],this->spacing[1],this->spacing[2]};
+      float gridSpacing[3] = {static_cast<float>(this->spacing[0]), static_cast<float>(this->spacing[1]), static_cast<float>(this->spacing[2])};
       if(this->spacing[0]<0)
          gridSpacing[0] *= -(this->inputReference->dx+this->inputFloating->dx)/2.f;
       if(this->spacing[1]<0)
